import os
import shlex

from fastapi import FastAPI, HTTPException
from fastapi.responses import FileResponse
from fastapi.staticfiles import StaticFiles
from pydantic import BaseModel
from onako.tmux_orchestrator import TmuxOrchestrator

app = FastAPI()
orch = TmuxOrchestrator()
static_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "static")


class CreateTaskRequest(BaseModel):
    prompt: str
    working_dir: str | None = None


class SendMessageRequest(BaseModel):
    message: str


@app.get("/")
def dashboard():
    return FileResponse(os.path.join(static_dir, "index.html"))


@app.get("/health")
def health():
    return {"status": "ok"}


@app.post("/tasks")
def create_task(req: CreateTaskRequest):
    command = f"claude {shlex.quote(req.prompt)}"
    task = orch.create_task(command, working_dir=req.working_dir, prompt=req.prompt)
    return task


@app.get("/tasks")
def list_tasks():
    return orch.list_tasks()


@app.get("/tasks/{task_id}")
def get_task(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    task = orch.tasks[task_id].copy()
    task["output"] = orch.get_output(task_id)
    return task


@app.get("/tasks/{task_id}/raw")
def get_task_raw(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    task = orch.tasks[task_id].copy()
    task["output"] = orch.get_raw_output(task_id)
    return task


@app.post("/tasks/{task_id}/message")
def send_message(task_id: str, req: SendMessageRequest):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    orch.send_message(task_id, req.message)
    return {"status": "sent"}


@app.delete("/tasks/{task_id}")
def delete_task(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    orch.kill_task(task_id)
    return {"status": "deleted"}


app.mount("/static", StaticFiles(directory=static_dir), name="static")



import re
import secrets
import shlex
import sqlite3
import subprocess
from datetime import datetime, timezone
from pathlib import Path


DB_PATH = Path.home() / ".onako" / "onako.db"


class TmuxOrchestrator:
    def __init__(self, session_name: str = "onako", db_path: Path | None = None):
        self.session_name = session_name
        self.db_path = db_path or DB_PATH
        self.tasks: dict[str, dict] = {}
        self._init_db()
        self._load_tasks()
        self._ensure_session()
        self.rediscover_tasks()

    def _ensure_session(self):
        result = subprocess.run(
            ["tmux", "has-session", "-t", self.session_name],
            capture_output=True,
        )
        if result.returncode != 0:
            subprocess.run(
                ["tmux", "new-session", "-d", "-s", self.session_name],
                check=True,
            )

    def _init_db(self):
        self.db_path.parent.mkdir(parents=True, exist_ok=True)
        conn = sqlite3.connect(self.db_path)
        conn.execute("""
            CREATE TABLE IF NOT EXISTS tasks (
                id TEXT PRIMARY KEY,
                prompt TEXT,
                status TEXT,
                started_at TEXT
            )
        """)
        conn.commit()
        conn.close()

    def _load_tasks(self):
        conn = sqlite3.connect(self.db_path)
        rows = conn.execute("SELECT id, prompt, status, started_at FROM tasks").fetchall()
        conn.close()
        for row in rows:
            self.tasks[row[0]] = {
                "id": row[0],
                "prompt": row[1],
                "status": row[2],
                "started_at": row[3],
            }

    def _save_task(self, task: dict):
        conn = sqlite3.connect(self.db_path)
        conn.execute(
            "INSERT OR REPLACE INTO tasks (id, prompt, status, started_at) VALUES (?, ?, ?, ?)",
            (task["id"], task["prompt"], task["status"], task["started_at"]),
        )
        conn.commit()
        conn.close()

    def _run_tmux(self, *args) -> subprocess.CompletedProcess:
        return subprocess.run(["tmux", *args], capture_output=True, text=True)

    def create_task(self, command: str, working_dir: str | None = None, prompt: str | None = None) -> dict:
        task_id = f"task-{secrets.token_hex(4)}"
        self._run_tmux(
            "new-window", "-t", self.session_name, "-n", task_id,
        )
        if working_dir:
            self._run_tmux(
                "send-keys", "-t", f"{self.session_name}:{task_id}",
                f"cd {shlex.quote(working_dir)}", "Enter",
            )
        self._run_tmux(
            "send-keys", "-t", f"{self.session_name}:{task_id}",
            command, "Enter",
        )
        task = {
            "id": task_id,
            "prompt": prompt or command,
            "status": "running",
            "started_at": datetime.now(timezone.utc).isoformat(),
        }
        self.tasks[task_id] = task
        self._save_task(task)
        return task

    def list_tasks(self) -> list[dict]:
        self._sync_task_status()
        return list(self.tasks.values())

    def get_output(self, task_id: str) -> str:
        raw = self.get_raw_output(task_id)
        cleaned = re.sub(r"\x1b\[[0-9;]*[a-zA-Z]", "", raw)
        return self._strip_claude_chrome(cleaned)

    @staticmethod
    def _strip_claude_chrome(text: str) -> str:
        lines = text.split("\n")
        # Strip from the bottom: Claude Code's TUI draws box-drawing chars,
        # the › prompt, and status lines like "accept edits on..."
        while lines:
            line = lines[-1].strip()
            if (
                not line
                or all(c in "─━╭╮╰╯│┃┌┐└┘├┤┬┴┼╋═║ ›❯▸▶" for c in line)
                or "accept edits" in line
                or "esc to interrupt" in line
                or "shift+tab to cycle" in line
                or "ctrl+" in line.lower()
                or line == "›"
            ):
                lines.pop()
            else:
                break
        return "\n".join(lines)

    def get_raw_output(self, task_id: str) -> str:
        result = self._run_tmux(
            "capture-pane", "-t", f"{self.session_name}:{task_id}",
            "-p", "-S", "-",
        )
        return result.stdout

    def send_message(self, task_id: str, message: str):
        self._run_tmux(
            "send-keys", "-t", f"{self.session_name}:{task_id}",
            "-l", message,
        )
        self._run_tmux(
            "send-keys", "-t", f"{self.session_name}:{task_id}",
            "Enter",
        )

    def kill_task(self, task_id: str):
        self._run_tmux("kill-window", "-t", f"{self.session_name}:{task_id}")
        if task_id in self.tasks:
            self.tasks[task_id]["status"] = "done"
            self._save_task(self.tasks[task_id])

    def _sync_task_status(self):
        result = self._run_tmux(
            "list-windows", "-t", self.session_name, "-F", "#{window_name}",
        )
        active_windows = set(result.stdout.strip().split("\n")) if result.stdout.strip() else set()
        for task_id, task in self.tasks.items():
            if task["status"] == "running" and task_id not in active_windows:
                task["status"] = "done"
                self._save_task(task)

    def rediscover_tasks(self):
        """Rediscover tasks from existing tmux windows on server restart."""
        result = self._run_tmux(
            "list-windows", "-t", self.session_name, "-F", "#{window_name}",
        )
        if not result.stdout.strip():
            return
        for window_name in result.stdout.strip().split("\n"):
            if window_name.startswith("task-") and window_name not in self.tasks:
                task = {
                    "id": window_name,
                    "prompt": "(rediscovered)",
                    "status": "running",
                    "started_at": None,
                }
                self.tasks[window_name] = task
                self._save_task(task)



import subprocess
import time
from pathlib import Path
import pytest
from onako.tmux_orchestrator import TmuxOrchestrator

SESSION_NAME = "onako-test"


@pytest.fixture(autouse=True)
def cleanup(tmp_path):
    """Kill test tmux session before and after each test."""
    subprocess.run(["tmux", "kill-session", "-t", SESSION_NAME], capture_output=True)
    yield
    subprocess.run(["tmux", "kill-session", "-t", SESSION_NAME], capture_output=True)


@pytest.fixture
def orch(tmp_path):
    return TmuxOrchestrator(session_name=SESSION_NAME, db_path=tmp_path / "test.db")


def test_create_task_returns_id(orch):
    task = orch.create_task("echo hello")
    assert task["id"].startswith("task-")
    assert task["status"] == "running"
    assert task["prompt"] == "echo hello"


def test_list_tasks_shows_created_task(orch):
    task = orch.create_task("echo hello")
    tasks = orch.list_tasks()
    assert any(t["id"] == task["id"] for t in tasks)


def test_get_output_captures_pane(orch):
    task = orch.create_task("echo hello-from-test")
    time.sleep(1)
    output = orch.get_output(task["id"])
    assert "hello-from-test" in output


def test_get_raw_output(orch):
    task = orch.create_task("echo raw-test")
    time.sleep(1)
    raw = orch.get_raw_output(task["id"])
    assert "raw-test" in raw


def test_send_message(orch):
    task = orch.create_task("cat")  # cat waits for stdin
    time.sleep(0.5)
    orch.send_message(task["id"], "hello-input")
    time.sleep(1)
    output = orch.get_output(task["id"])
    assert "hello-input" in output


def test_kill_task(orch):
    task = orch.create_task("sleep 999")
    orch.kill_task(task["id"])
    tasks = orch.list_tasks()
    assert not any(t["id"] == task["id"] and t["status"] == "running" for t in tasks)


def test_create_multiple_tasks(orch):
    t1 = orch.create_task("echo one")
    t2 = orch.create_task("echo two")
    tasks = orch.list_tasks()
    ids = [t["id"] for t in tasks]
    assert t1["id"] in ids
    assert t2["id"] in ids
    assert t1["id"] != t2["id"]



import pytest
from fastapi.testclient import TestClient


@pytest.fixture
def client():
    import importlib
    from onako import server
    importlib.reload(server)
    client = TestClient(server.app)
    yield client
    response = client.get("/tasks")
    for task in response.json():
        client.delete(f"/tasks/{task['id']}")


def test_health(client):
    r = client.get("/health")
    assert r.status_code == 200
    assert r.json()["status"] == "ok"


def test_create_task(client):
    r = client.post("/tasks", json={"prompt": "echo api-test"})
    assert r.status_code == 200
    assert r.json()["id"].startswith("task-")


def test_list_tasks(client):
    client.post("/tasks", json={"prompt": "echo one"})
    client.post("/tasks", json={"prompt": "echo two"})
    r = client.get("/tasks")
    assert r.status_code == 200
    assert len(r.json()) >= 2


def test_get_task(client):
    create = client.post("/tasks", json={"prompt": "echo detail-test"})
    task_id = create.json()["id"]
    import time
    time.sleep(1)
    r = client.get(f"/tasks/{task_id}")
    assert r.status_code == 200
    assert r.json()["id"] == task_id
    assert "output" in r.json()


def test_get_task_raw(client):
    create = client.post("/tasks", json={"prompt": "echo raw-api-test"})
    task_id = create.json()["id"]
    import time
    time.sleep(1)
    r = client.get(f"/tasks/{task_id}/raw")
    assert r.status_code == 200
    assert "output" in r.json()


def test_send_message(client):
    create = client.post("/tasks", json={"prompt": "cat"})
    task_id = create.json()["id"]
    r = client.post(f"/tasks/{task_id}/message", json={"message": "hello"})
    assert r.status_code == 200


def test_delete_task(client):
    create = client.post("/tasks", json={"prompt": "sleep 999"})
    task_id = create.json()["id"]
    r = client.delete(f"/tasks/{task_id}")
    assert r.status_code == 200


def test_get_nonexistent_task(client):
    r = client.get("/tasks/task-nonexistent")
    assert r.status_code == 404



from click.testing import CliRunner
from onako.cli import main


def test_status_reports_something():
    runner = CliRunner()
    result = runner.invoke(main, ["status"])
    assert result.exit_code == 0
    assert "Onako server:" in result.output


def test_stop_completes():
    runner = CliRunner()
    result = runner.invoke(main, ["stop"])
    assert result.exit_code == 0
    assert "not running" in result.output or "stopped" in result.output



from click.testing import CliRunner
from onako.cli import main


def test_version():
    runner = CliRunner()
    result = runner.invoke(main, ["version"])
    assert result.exit_code == 0
    assert "0.2.0" in result.output


def test_serve_help():
    runner = CliRunner()
    result = runner.invoke(main, ["serve", "--help"])
    assert result.exit_code == 0
    assert "--host" in result.output
    assert "--port" in result.output
    assert "--background" in result.output
    assert "--dir" in result.output



import os
import platform
import shutil
import subprocess
import sys
from pathlib import Path

import click

ONAKO_DIR = Path.home() / ".onako"
LOG_DIR = ONAKO_DIR / "logs"


@click.group()
def main():
    """Onako — Dispatch and monitor Claude Code tasks from your phone."""
    pass


@main.command()
def version():
    """Print the version."""
    from onako import __version__
    click.echo(f"onako {__version__}")


@main.command()
@click.option("--host", default="0.0.0.0", help="Host to bind to.")
@click.option("--port", default=8787, type=int, help="Port to bind to.")
@click.option("--dir", "working_dir", default=None, type=click.Path(exists=True), help="Working directory for tasks (default: current directory).")
@click.option("--background", is_flag=True, help="Run as a background service (launchd/systemd).")
def serve(host, port, working_dir, background):
    """Start the Onako server."""
    working_dir = str(Path(working_dir).resolve()) if working_dir else os.getcwd()

    if background:
        _start_background(host, port, working_dir)
        return

    _check_prerequisites()

    os.environ["ONAKO_WORKING_DIR"] = working_dir

    from onako import __version__
    click.echo(f"Onako v{__version__}")
    click.echo(f"Starting server at http://{host}:{port}")
    click.echo(f"Working directory: {working_dir}")
    click.echo()

    import uvicorn
    from onako.server import app
    uvicorn.run(app, host=host, port=port)


@main.command()
def stop():
    """Stop the background Onako service."""
    system = platform.system()
    if system == "Darwin":
        plist_path = Path.home() / "Library" / "LaunchAgents" / "com.onako.server.plist"
        if plist_path.exists():
            uid = os.getuid()
            result = subprocess.run(
                ["launchctl", "bootout", f"gui/{uid}", str(plist_path)],
                capture_output=True,
            )
            if result.returncode != 0:
                subprocess.run(["launchctl", "unload", str(plist_path)], capture_output=True)
            plist_path.unlink()
            click.echo("Onako service stopped and removed.")
        else:
            click.echo("Onako service is not running.")
    elif system == "Linux":
        unit_path = Path.home() / ".config" / "systemd" / "user" / "onako.service"
        if unit_path.exists():
            subprocess.run(["systemctl", "--user", "disable", "--now", "onako"])
            unit_path.unlink()
            subprocess.run(["systemctl", "--user", "daemon-reload"])
            click.echo("Onako service stopped and removed.")
        else:
            click.echo("Onako service is not running.")
    else:
        click.echo(f"Not supported on {system}.")


@main.command()
def status():
    """Check if Onako is running."""
    import urllib.request
    try:
        r = urllib.request.urlopen("http://127.0.0.1:8787/health", timeout=2)
        data = r.read().decode()
        if '"ok"' in data:
            click.echo("Onako server: running")
            click.echo("  URL: http://127.0.0.1:8787")
        else:
            click.echo("Onako server: not responding correctly")
    except Exception:
        click.echo("Onako server: not running")


def _start_background(host, port, working_dir):
    """Install and start Onako as a background service."""
    system = platform.system()
    onako_bin = shutil.which("onako")
    if not onako_bin:
        click.echo("Error: 'onako' command not found on PATH.", err=True)
        sys.exit(1)

    LOG_DIR.mkdir(parents=True, exist_ok=True)

    # Build PATH that includes dirs for tmux and claude
    path_dirs = set()
    for cmd in ["tmux", "claude"]:
        p = shutil.which(cmd)
        if p:
            path_dirs.add(str(Path(p).parent))
    path_dirs.update(["/usr/local/bin", "/usr/bin", "/bin"])
    path_value = ":".join(sorted(path_dirs))

    if system == "Darwin":
        _install_launchd(onako_bin, host, port, working_dir, path_value)
    elif system == "Linux":
        _install_systemd(onako_bin, host, port, working_dir, path_value)
    else:
        click.echo(f"Background mode is not supported on {system}. Run 'onako serve' manually.", err=True)
        sys.exit(1)


def _install_launchd(onako_bin, host, port, working_dir, path_value):
    from importlib.resources import files
    tpl = files("onako").joinpath("templates", "com.onako.server.plist.tpl").read_text()
    plist = tpl.format(
        onako_bin=onako_bin,
        host=host,
        port=port,
        working_dir=working_dir,
        log_dir=LOG_DIR,
        path_value=path_value,
    )
    plist_path = Path.home() / "Library" / "LaunchAgents" / "com.onako.server.plist"
    plist_path.parent.mkdir(parents=True, exist_ok=True)

    # Unload existing service if present
    uid = os.getuid()
    subprocess.run(
        ["launchctl", "bootout", f"gui/{uid}", str(plist_path)],
        capture_output=True,
    )

    plist_path.write_text(plist)

    # Register and start the service
    result = subprocess.run(
        ["launchctl", "bootstrap", f"gui/{uid}", str(plist_path)],
        capture_output=True, text=True,
    )
    if result.returncode != 0:
        subprocess.run(["launchctl", "load", str(plist_path)], check=True)

    # kickstart forces the service to run now (bootstrap alone may not start it)
    subprocess.run(
        ["launchctl", "kickstart", f"gui/{uid}/com.onako.server"],
        capture_output=True,
    )

    click.echo(f"Onako running in background at http://{host}:{port}")
    click.echo(f"  Working directory: {working_dir}")
    click.echo(f"  Logs: {LOG_DIR}")
    click.echo()
    click.echo("If macOS blocks this service, allow it in:")
    click.echo("  System Settings > General > Login Items & Extensions")


def _install_systemd(onako_bin, host, port, working_dir, path_value):
    from importlib.resources import files
    tpl = files("onako").joinpath("templates", "onako.service.tpl").read_text()
    unit = tpl.format(
        onako_bin=onako_bin,
        host=host,
        port=port,
        working_dir=working_dir,
        path_value=path_value,
    )
    unit_path = Path.home() / ".config" / "systemd" / "user" / "onako.service"
    unit_path.parent.mkdir(parents=True, exist_ok=True)
    unit_path.write_text(unit)
    subprocess.run(["systemctl", "--user", "daemon-reload"], check=True)
    subprocess.run(["systemctl", "--user", "enable", "--now", "onako"], check=True)
    click.echo(f"Onako running in background at http://{host}:{port}")
    click.echo(f"  Working directory: {working_dir}")


def _check_prerequisites():
    """Check that tmux and claude are installed."""
    tmux_path = shutil.which("tmux")
    if not tmux_path:
        click.echo("Error: tmux is not installed.", err=True)
        click.echo("Install it with: brew install tmux (macOS) or apt install tmux (Linux)", err=True)
        sys.exit(1)
    click.echo(f"  tmux: {tmux_path}")

    claude_path = shutil.which("claude")
    if not claude_path:
        click.echo("Warning: claude CLI not found on PATH.", err=True)
        click.echo("Install Claude Code from: https://docs.anthropic.com/en/docs/claude-code", err=True)
    else:
        click.echo(f"  claude: {claude_path}")
